# -*- coding: utf-8 -*-
# Copyright 2019 Ildar.Shirshov <ildar-shirshov@ya.ru>
#  
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import random
import string


def make_random_string(pack: str = None, n: int = 15) -> str:
    """
    Function generates random string which
    specific string cases.
    :param pack: alphabet
    :param n: length of generated string
    :return: as is
    """
    if not pack:
        pack = string.ascii_uppercase + string.digits + string.ascii_lowercase

    for _ in range(n):
        yield random.SystemRandom().choice(pack)



# -*- coding: utf-8 -*-
# Copyright 2019 Ildar.Shirshov <ildar-shirshov@ya.ru>
#  
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import inspect

from functools import wraps
from functools import partial

import ujson

from aiohttp import web
from webargs import aiohttpparser
from webargs import dict2schema

ROUTE_TABLE = []
JSON_RESPONSE = partial(web.json_response, dumps=ujson.dumps)

GET = 1 << 0
POST = 1 << 1


def __to_camel(method_name: str) -> str:
    """
    Function converts to camel
    case specific method name.

    First version of naming:
        <get|set>_your_action -> getYourAction

    Second version of naming:
        _<get|set>            -> get

    :param method_name: As is
    :return: str
    """

    tmp = method_name[:]

    if tmp.startswith('_'):
        tmp = tmp[-(len(tmp) - 1):]

    return "".join(
        w.lower() if i == 0 else w.title() for i, w in enumerate(tmp.split('_'))
    )


def __url_path_format(version_id: str, resource_name: str, method_name: str) -> str:
    """
    Function makes the method path
    using specific format.
    :param version_id: Version of API
    :param method_name: Name of method
    :param resource_name: Name of resource
    :return: str
    """
    return f"/{version_id}/{resource_name}.{method_name}"


def make_route(input_args: dict = None, output_args: dict = None,
               locations: tuple = None, validate: object = None,
               path: str = None, auth_required: callable = None,
               method_types: int = None):
    """
    Functions makes the route for
    wrapped function.
    :param input_args: Map of incoming data scheme
    :param output_args: Map of output scheme
    :param locations: Location of incoming data ('form', 'data', 'json')
    :param validate: Validator of full scheme
    :param path: Stringify path of decorated async function
    :param auth_required: This functions checks credential information inside request headers
    :param method_types: This flags indicates about methods for specific endpoint
    :return: callable
    """
    if not isinstance(method_types, int):
        method_types = GET | POST

    # Prepare dumper for outgoing data
    # if not (isinstance(output_args, dict) or isinstance(output_args, list)):
    #    output_args = {'result': fields.List(fields.Str(), missing=[])}
    if isinstance(output_args, dict):
        schema = dict2schema(output_args)()
    else:
        schema = output_args()

    # Checking locations of data
    # for incoming request
    if not isinstance(locations, tuple):
        locations = ('json', 'querystring')

    def wrapped(fn_: callable):
        # Create the wrapper for original
        # async coroutine function

        @wraps(fn_)
        async def coro_wrapper(request: web.Request):
            # First checking authorization
            auth_data = None
            if inspect.iscoroutinefunction(auth_required):
                auth_data = await auth_required(request)

            # Parsing incoming data using parser and
            # scheme of input arguments
            parsed_data_map = dict()
            if input_args:
                parsed_data_map = await aiohttpparser.parser.parse(
                    input_args, request, locations, validate
                )

            # Injecting data from request's header
            # data
            if auth_data:
                parsed_data_map = {**parsed_data_map, **auth_data}

            # Waiting the result from original function
            # and dumping it
            data = await fn_(**{**parsed_data_map, **{'request': request}})

            if isinstance(data, (web.FileResponse, web.WebSocketResponse, web.Response)):
                return data

            # 1) Validating outgoing data scheme
            # 2) Serialize verified data
            if data is None:
                data = {}
            else:
                if isinstance(output_args, dict):
                    if not isinstance(data, tuple):
                        data = (data,)

                    data = dict(zip(output_args.keys(), data))

            return JSON_RESPONSE(
                schema.dump(data)
            )

        # Last thing is dispatch the route of
        # aiohttp coroutine and it endpoint
        endpoint_path = path
        if not isinstance(endpoint_path, str):
            # First step is extract
            # name of called module
            frm = inspect.stack()[1]
            mod = inspect.getmodule(frm[0])

            # This module name should
            # exclude from global path
            root_file = __name__

            # The full path to wrapped
            # resource endpoint
            resource_path = mod.__name__
            resource_path = resource_path.replace(root_file, '')
            resource_path = resource_path.replace('.', '/')

            # Extract endpoint name
            method_name = __to_camel(fn_.__name__)
            endpoint_path = "/{0}.{1}".format(resource_path, method_name)

        # Add coroutine wrapper inside the webapp
        if (method_types & GET) == GET:
            ROUTE_TABLE.append(
                web.get(endpoint_path, coro_wrapper, allow_head=False)
            )
        if (method_types & POST) == POST:
            ROUTE_TABLE.append(
                web.post(endpoint_path, coro_wrapper)
            )

        return coro_wrapper

    return wrapped



# -*- coding: utf-8 -*-
# Copyright 2019 Ildar.Shirshov <ildar-shirshov@ya.ru>
#  
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

__version__ = "0.0.1a2"
__doc__ = """
"""
__author__ = "Ildar Shirshov"
__license__ = "Apache 2.0"

from . import routes
from . import adaptor
from . import generators



# This is purely the result of trial and error.
import os
import sys
import codecs
import subprocess

from setuptools import setup
from setuptools import find_packages

import aiowrpr

INSTALL_REQUIRES = [
    'aiodns==2.0.0',
    'aiohttp[speedups]==3.6.2',
    'async-timeout==3.0.1',
    'attrs==19.3.0',
    'brotlipy==0.7.0',
    'cchardet==2.1.5',
    'cffi==1.13.2',
    'chardet==3.0.4',
    'idna==2.8',
    'marshmallow==3.3.0',
    'multidict==4.7.1',
    'pycares==3.1.0',
    'pycparser==2.19',
    'ujson==1.35',
    'webargs==5.5.2',
    'yarl==1.4.2'
]


# Conditional dependencies:
if sys.version_info < (3, 5) or sys.version_info > (3, 8):
    sys.exit(
        f"Sorry, Python {'.'.join(map(str, sys.version_info[:3]))} is not supported"
    )


def long_description():
    with codecs.open('README.md', encoding='utf8') as f_:
        return f_.read()


# Fetch version from git tags, and write to version.py.
# Also, when git is not available (PyPi package), use stored version.py.
VERSION_PY = os.path.join(os.path.dirname(__file__), 'version.py')
try:
    VERSION_GIT = str(subprocess.check_output(["git", "describe", "--tags"]).rstrip(), 'utf-8')
except Exception as _:
    with open(VERSION_PY, 'r') as fh:
        VERSION_GIT = open(VERSION_PY).read().strip().split('=')[-1].replace('"', '')

VERSION_MSG = "# Do not edit this file, pipeline versioning is governed by git tags"
with open(VERSION_PY, 'w') as fh:
    fh.write(f'{VERSION_MSG}{os.linesep}__version__={VERSION_GIT}')


setup(
    name='aiowrpr',
    version=VERSION_GIT,
    description=aiowrpr.__doc__.strip(),
    long_description=long_description(),
    url='https://github.com/ishirshov/aiowrpr',
    download_url='https://github.com/ishirshov/aiowrpr',
    author=aiowrpr.__author__,
    author_email='ildar.shirshov@gmail.com',
    license=aiowrpr.__license__,
    packages=find_packages(),
    entry_points={
        'console_scripts': [
            'http = httpie.__main__:main',
            'https = httpie.__main__:main',
        ],
    },
    install_requires=INSTALL_REQUIRES,
    classifiers=[
        'Development Status :: 1 - Planning',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: BSD License',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Software Development',
        'Topic :: System :: Networking',
        'Topic :: Terminals',
        'Topic :: Text Processing',
        'Topic :: Utilities'
    ],
)


# -*- coding: utf-8 -*-
# Copyright 2019 Ildar.Shirshov <ildar-shirshov@ya.ru>
#  
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import unicodedata

from functools import partial
from typing import Optional, Union

import asyncio
import ujson

from aiohttp import ClientSession
from aiohttp import hdrs
from aiohttp.web_request import Request
from webargs import aiohttpparser
from webargs import core

JSONCompatible = Union[str, dict, list, set]


async def make_sync_call(fn_: partial) -> object:
    """
    Function makes async call of synchronous
    function.
    :param fn_: partial object
    :return: some data
    """
    loop = asyncio.get_event_loop()
    return await loop.run_in_executor(None, fn_)


async def loads_json(
    req: Request, unicode_normalizing_form: Optional[str] = 'NFKC'
) -> Optional[JSONCompatible]:
    """
    Function loads json data
    from response data (text)
    using ujson module.
    :param req: Incoming request
    :param unicode_normalizing_form: as is
    :return: JSON data structure
    """
    raw_text = await req.text()

    if unicode_normalizing_form:
        raw_text = unicodedata.normalize(unicode_normalizing_form, raw_text)

    return ujson.loads(raw_text) if raw_text else None


async def make_request(
    url: str, params=None, headers=None, method: str = hdrs.METH_GET
) -> Optional[JSONCompatible]:
    """
    Function makes the request
    with specific url and data query.
    :param url: base url
    :param params: parameter for query
    :param headers: as is
    :param method: possible methods POST, GET
    :return: json structure data
    """
    if params is None:
        params = dict()

    if headers is None:
        headers = dict()

    kwargs = {
        hdrs.METH_POST: {'json': params, 'headers': headers},
        hdrs.METH_GET: {'params': params, 'headers': headers},
    }[method]

    async with ClientSession() as session:
        async with session.request(method, url, **kwargs) as resp:
            return await loads_json(resp)


class CachedAIOHttpParser(aiohttpparser.AIOHTTPParser):
    """aiohttp request argument parser."""

    __location_map__ = dict(
        match_info='parse_match_info',
        **core.Parser.__location_map__
    )

    def parse_files(self, req: Request, name: str, field: aiohttpparser.Field) -> None:
        raise NotImplementedError(
            "parse_files is not implemented. You may be able to use parse_form for "
            "parsing upload data."
        )

    async def parse_json(self, req, name, field):
        """Pull a json value from the request."""
        if not self._cache:
            if not (req.can_read_body and aiohttpparser.is_json_request(req)):
                return core.missing

        json_data = self._cache.get('json')
        if json_data is None:
            self._cache["json"] = json_data = await loads_json(req)

        return core.get_value(json_data, name, field, allow_many_nested=True)


aiohttpparser.parser = CachedAIOHttpParser()
aiohttpparser.use_args = aiohttpparser.parser.use_args
aiohttpparser.use_kwargs = aiohttpparser.parser.use_kwargs
